# kong-plugin-enterprise-degraphql

De-GraphQL is a plugin to create your own REST API on top of Kong by querying a
GraphQL service. Tecnology moves forward, but sometimes you still need to
support backwards.

## Usage

De-GraphQL needs a graphql endpoint to query. As an example, we are going to
build a REST API around https://swapi-graphql.eskerda.now.sh

### Start kong, and add the graphql endpoint as a service

```
KONG_PLUGINS=degraphql gojira up --volume $(realpath .):/kong-plugin
gojira shell
kong migrations bootstrap
kong start
```

```bash
http -f :8001/services name=example host=swapi-graphql.eskerda.now.sh protocol=https port=443
http -f :8001/services/example/routes hosts=example.com
# Test that it works
http :8000 Host:example.com query='{ allPeople { people { name } } }'
```

### Configure the plugin on the service

The plugin takes over the service. From this point on, the service represents
our REST api and not the graphql endpoint itself. Thus, it will return a 404 Not Found
if no degraphql routes have been configured.

```bash
http :8001/services/example/plugins name=degraphql
```

### Configure routes on the plugin

```
http :8001/services/example/degraphql/routes uri='/people' query='{ allPeople { people { id name } } }'
http :8001/services/example/degraphql/routes uri='/planets' query='{ allPlanets { planets { id name climates terrains diameter } } }'
```

Note that GET, POST and URL parameters are available through graphql variable
scoping:

```
http :8001/services/example/degraphql/routes uri='/planets/:id' query='query one_planet ($id: ID) { planet(id: $id) { id name } }'
```

The same parameters are also allowed inside the graphql query as template variables:

```
http :8001/services/example/degraphql/routes uri='/people/:id' query='{ person (id: "{{id}}") { name {{fields}} } }'
```

```
http :8000/people Host:example.com
http :8000/planets Host:example.com
http :8000/planets/cGxhbmV0czo2MQ== Host:example.com
http :8000/people/cGVvcGxlOjE=?fields=gender,mass Host:example.com
```
